/*!
  \file
  \brief 接続用のスレッド

  \author Satofumi Kamimura
*/

#include "Connect_thread.h"
#include "Connection.h"

using namespace hrk;
using namespace std;


struct Connect_thread::pImpl
{
    Connect_thread* thread_;
    hrk::Lidar& lidar_;
    Connection* connection_;
    string device_or_address_;
    long baudrate_or_port_;
    Lidar::connection_t type_;


    pImpl(Connect_thread* thread, Lidar& lidar)
        : thread_(thread), lidar_(lidar)
    {
    }
};


Connect_thread::Connect_thread(hrk::Lidar& lidar)
    : pimpl(new pImpl(this, lidar))
{
}


Connect_thread::~Connect_thread(void)
{
}


void Connect_thread::run(void)
{
    bool ret = false;
    const string series_name = pimpl->lidar_.sensor_series_name();

    if (pimpl->connection_) {
        ret = pimpl->lidar_.open(pimpl->connection_);

    } else if (!pimpl->device_or_address_.empty()) {
        ret = pimpl->lidar_.open(pimpl->device_or_address_.c_str(),
                                 pimpl->baudrate_or_port_, pimpl->type_);
    }

    if (ret) {
        emit connected();
    } else {
        emit connect_failed(pimpl->lidar_.what());
    }
}


void Connect_thread::set_open_setting(hrk::Connection* connection)
{
    pimpl->connection_ = connection;

    pimpl->device_or_address_.clear();
}


void Connect_thread::set_open_setting(const std::string& device_or_address,
                                      long baudrate_or_port,
                                      hrk::Lidar::connection_t type)
{
    pimpl->device_or_address_ = device_or_address;
    pimpl->baudrate_or_port_ = baudrate_or_port;
    pimpl->type_ = type;

    pimpl->connection_ = NULL;
}
