﻿/*!
  \file
  \brief トークンまわりの関数群

  \author Satofumi Kamimura
*/

#include <cstdlib>
#include "token_utils.h"

using namespace hrk;
using namespace std;


std::string hrk::trim(const std::string& token)
{
    const char* white_spaces = " \r\n\t";
    string::size_type first_index = token.find_first_not_of(white_spaces);
    string::size_type last_index = token.find_last_not_of(white_spaces);

    // white_space しかないときは、戻る
    if ((first_index == string::npos) && (last_index == string::npos)) {
        return "";
    }

    string::size_type length = (last_index == string::npos) ?
        string::npos : last_index - first_index + 1;
    return token.substr(first_index, length);
}


std::vector<string> hrk::split(const std::string& line,
                               const char* separator)
{
    vector<string> tokens;

    bool is_terminated = false;
    string::size_type first_index = 0;
    string::size_type next_first_index = 0;
    do {
        string::size_type last_index =
            line.find_first_of(separator, first_index);

        string::size_type length;
        if (last_index == string::npos) {
            // first_index から最後までを最後の token として扱う
            length = string::npos;
            is_terminated = true;
        } else {
            length = last_index - first_index;
            next_first_index = last_index + 1;
        }

        string token = line.substr(first_index, length);
        tokens.push_back(trim(token));

        first_index = next_first_index;
    } while (!is_terminated);

    return tokens;
}


std::string hrk::join(std::vector<std::string>& tokens, const char* separator)
{
    string line;
    for (vector<string>::const_iterator it = tokens.begin();
         it != tokens.begin(); ++it) {
        line += *it;
        if ((it + 1) == tokens.end()) {
            line += separator;
        }
    }
    return line;
}


void hrk::remove_empty_tokens(std::vector<std::string>& tokens)
{
    vector<string> replaced_tokens;
    for (vector<string>::const_iterator it = tokens.begin();
         it != tokens.end(); ++it) {
        if (it->empty()) {
            continue;
        }
        replaced_tokens.push_back(*it);
    }
    tokens.swap(replaced_tokens);
}


SizeF hrk::decode_size(const std::string& size_line)
{
    vector<string> tokens = split(size_line, "x");
    if (tokens.size() != 2) {
        return SizeF(0, 0);
    }
    double width = atof(tokens[0].c_str());
    double height = atof(tokens[1].c_str());

    return SizeF(width, height);
}


PointF hrk::decode_point(const std::string& point_line)
{
    char* line = const_cast<char*>(point_line.c_str());
    char* p;
    double x = strtol(line, &p, 10);
    double y = strtol(p, NULL, 10);
    return PointF(x, y);
}
